{
********************************************
RFID.spin                              V1.1
********************************************

Version 1.0 - Initial release

Version 1.1 - Added Serial timeout mechanism to allow RFID reader to turn OFF.
            - As stated in the RFID documentation, it is not advised to leave
              the unit in a continuous READ mode.

              Note: If a TAG is in the RFID reader's field of view, READ mode
                    will remain active(red LED on).  If a TAG is out of the
                    RFID reader's field of view, the RFID Reader will cycle
                    Enabled(red LED)/Disabled(green LED) for a duration of
                    about 1 second. 

Description:
                                                                     
This object runs in a separate cog to constantly monitor a Parallax RFID reader. The
ASCII 10-byte unique RFID is placed in a buffer that you specify the starting byte
address of 10 contiguous ASCII byte locations.  Starting the object, automatically
enables the RFID reader.  Likewise Stopping the object will disable the RFID reader.

Size:
Program  - 50 longs
Variable - 16 longs

}

CON
  
  _1ms  = 1_000_000 /     1_000     ' Divisor for 1 ms for simulated reader

VAR
  long    Stack[15],cog
  long clkcycles   ' for simulated reader 

PUB stop
    if cog
       cogstop(cog~ -  1)

PUB start(RXPin,EnablePin,RFIDbuffer)
    stop
    result := cog := cognew(ReadTAG(RXPin,EnablePin,RFIDbuffer),@Stack)+ 1

PRI ReadTAG(RXPin, EnablePin, RFIDAddress)|i

  ' ----------------  replacing proc for simulated reader -------------------
  ' For just fill RFIDbuffer with simulated badge ID
  ' port 0 high for send badge ID 1  
  ' port 1 high for send badge ID 2
  ' button 1 between Port 0 and VDD with a pulldown resistor
  ' button 2 between Port 1 and VDD with a pulldown resistor  

  dira[0]~    ' button 1
  dira[1]~    ' button 2

  repeat 20
    !outa[16]
    waitcnt(5_000_000 + cnt)
    'pause1ms(500)
  
  repeat
  
    if ina[0] == 1              ' button 1 between Port 0 and VDD with a pulldown resistor 
      pause1ms(20)              ' debouncing proc
      repeat while ina[0] == 1  '    "
      pause1ms(20)              '    "
      
      repeat i from 0 to 9
        byte[RFIDAddress+i] := byte[@badge1+i]
        pause1ms(5)              ' 5 mSec for 2400 baud

    if ina[1] == 1               ' button 2 between Port 1 and VDD with a pulldown resistor
      pause1ms(20)               ' debouncing proc
      repeat while ina[1] == 1   '    "
      pause1ms(20)               '    "
      
      repeat i from 0 to 9
        byte[RFIDAddress+i] := byte[@badge2+i]
        pause1ms(5)              ' 5 mSec for 2400 baud

   '--------------------------------------------------------------
        

  {{  Original reading from RFID reader
  
    dira[EnablePin]~~                                   ' Set Enable pin as an OUTPUT
    repeat
      i~                                                ' clear i ; i := 0
      repeat until result == {StartByte}$0A             ' Look for RFID Start byte
        result := Serin(RXPin)                          ' read RFID byte
        if result == false                              ' If result = false then a Serial
                                                        ' timeout occurred give reader a
                                                        ' break and Disable.
           outa[EnablePin]~~                            ' Disable RFID ; Make Enable pin
                                                        ' HIGH
           waitcnt(cnt + clkfreq)                       ' Wait for 1 sec
           outa[EnablePin]~                             ' Enable RFID ; Make Enable pin
                                                        ' LOW
      repeat
        result := Serin(RXPin)                          ' read RFID byte
        if result == {StopByte}$0D                      ' Look for RFID Stop byte
           quit
        byte[RFIDAddress+i] := result                   ' Update RFID buffer
        i := i + 1 <# 9                                 ' increment RFID buffer pointer ;
                                                        ' limit to 9 max

  }}
  
PRI Serin(RXPin)|i                                 
    dira[RXPin]~                                        ' Set as input
    waitpeq(|< RXPin, |< RXPin, 0)                      ' Wait for idle
    i := cnt + clkfreq                                  ' Set timeout interval to 1 Second
    result := true                                         
    repeat while ina[RXPin] == 1                        ' Wait for Start bit
      if cnt  > i                                       ' timeout after 1 second
         result := false                                ' Set Result = false as a timeout
                                                        ' flag
         quit
    if result == true                                   ' Proceed if there was no timeout
       waitcnt (cnt + clkfreq / 1600{2/3 Baud})         ' Pause 1.5 bit time
       result := ina[RXPin]                             ' Read LSB
       Repeat i from 1 to 7                             ' Read next data bit
         waitcnt (cnt + clkfreq / 2400{Baud})           ' Wait until center of next bit
         result |= ina[RXPin] << i                      ' Read next bit, shift and store



PUB pause1ms(period)  '  for simulated reader

'' Pause execution for period (in units of 1 ms).

  clkcycles := ((clkfreq / _1ms * period) - 4296) #> 381     ' Calculate 1 ms time unit
  waitcnt(clkcycles + cnt)                                   ' Wait for designated time

DAT ' for simulated reader
   badge1  byte "0ABCDEFGHI"
   badge2  byte "0KLMNOPQRS"        